﻿using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using System.Security.Principal;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;
using System.Web.Security;
using DEOCIO.Models;
using DEOCIO.DataModel.TablasMaestras.msStateProvince;
using DEOCIO.DataModel.Products;

// Controlador Account
namespace DEOCIO.Controllers
{
    public class AccountController : Controller
    {
        // Definimos los servicios para interactuar con el modelo de datos
        public IFormsAuthenticationService FormsService { get; set; }
        public IMembershipService MembershipService { get; set; }
        public ProductService ProductServices;


        protected override void Initialize(RequestContext requestContext)
        {
            // Inicializamos los servicios.
            if (FormsService == null) { FormsService = new FormsAuthenticationService(); }
            if (MembershipService == null) { MembershipService = new AccountMembershipService(); }
            if (ProductServices == null) { ProductServices = new ProductModels(); }

            base.Initialize(requestContext);
        }

        //**************************************************************************
        //Nombre: LogOn()
        //Descripción: Devuelve la vista LogOn() para validar al usuario
        //@params []
        //**************************************************************************
        public ActionResult LogOn()
        {
            return View();
        }

        //**************************************************************************
        //Nombre:  [HttpPost]LogOn()
        //Descripción: Recibe los parámetros del formulario y valida al usuario
        //@params []
        //**************************************************************************
        [HttpPost]
        public ActionResult LogOn(LogOnModel model, string returnUrl)
        {
            // Valida el modelo de datos
            if (ModelState.IsValid)
            {
                // Valida al usuario
                string returncode = MembershipService.ValidateUser(model.UserName, model.Password);
                int statuscode = -1;

                bool isNum = Int32.TryParse(returncode, out statuscode);

                if (isNum)
                {
                    // Crea la sessión del usuario
                    FormsService.SignIn(model.UserName, model.RememberMe);
                    
                    // Si se trata de una URL local redireccionamos
                    if (Url.IsLocalUrl(returnUrl))
                    {
                        return Redirect(returnUrl);
                    }
                    else
                    {
                        return RedirectToAction("Index", "Home");
                    }
                }
                else
                {
                    // Mostramos los errores.
                    ModelState.AddModelError("",returncode);
                }
            }
            // Si la función ha llegado aquí, algo ha falldo, volvemos a enseñar el formulario.
            return View(model);
        }

        //**************************************************************************
        //Nombre:  LogOff()
        //Descripción: Cierra la sesión del usuario
        //@params []
        //**************************************************************************
        public ActionResult LogOff()
        {
            FormsService.SignOut();
            // Redirecciona a la página inicial
            return RedirectToAction("Index", "Home");
        }

        //**************************************************************************
        //Nombre:  Register()
        //Descripción: Muestra el formulario de registro
        //@params []
        //**************************************************************************
        public ActionResult Register()
        {
            ViewBag.PasswordLength = MembershipService.MinPasswordLength;
            
            // Carga los datos de las províncias.
            var selectlist = new SelectList(MembershipService.msStateProvince_All(), "StateProvinceID", "Name", "S");
            ViewData["msProvincias"] = selectlist;

           // Carga los datos de las categorías.
            List<dtProductSubCategory> Categories = ProductServices.ProductSubCategory_GetAll().ToList();
            MultiSelectList cats = new MultiSelectList(Categories, "ProductSubCategoryId", "Name", Categories);
            ViewData["cats"] = cats;
            
            // Devuelve la vista
            return View();
        }

        //**************************************************************************
        //Nombre: [HttpPost]Register()
        //Descripción: Recoge los datos del formulario por POST, los valida e intenta crear al usuario.
        //@params [RegisterModel model]
        //**************************************************************************
        [HttpPost]
        public ActionResult Register(RegisterModel model)
        {
            // Hay que aceptar el contrato antes de continuar
            if (!model.Contrato)
                ModelState.AddModelError("Contrato", "Debe aceptar el contrato antes de continuar...");

            // Valida el modelo de datos
            if (ModelState.IsValid)
            {
                // Intenta registrar al usuario.
                string createStatus = MembershipService.CreateUser(model.Sufijo, model.Name, model.PrimerApellido, model.SegundoApellido, 
                                                                                   model.DNI, model.Email, model.Password, model.Direccion, model.CP,
                                                                                   model.Poblacion, model.Provincia, model.Telefono, model.Movil,
                                                                                   model.Contrato, model.RecibirInfo, 1, model.Categorias);

                // Si el resultado es 1 -> Usuario registrado correctamente.
                if (createStatus == "1")
                {
                    // Iniciamos la sesión del usuario.
                    FormsService.SignIn(model.Email, false /* createPersistentCookie */);
                    return RedirectToAction("Index", "Home");
                }
                else
                {
                    // Mostramos los errores en la validación del modelo
                    ModelState.AddModelError("", createStatus);
                }
            }

            // Si la función ha llegado aquí es que algo ha fallado, volvemos a mostrar el formulario
            ViewBag.PasswordLength = MembershipService.MinPasswordLength;
            
            // Cargamos los datos de las províncias
            var selectlist = new SelectList(MembershipService.msStateProvince_All(), "StateProvinceID", "Name", model.Provincia);
            ViewData["msProvincias"] = selectlist;

            // Cargamos los datos de las categorias
            List<dtProductSubCategory> Categories = ProductServices.ProductSubCategory_GetAll().ToList();
            MultiSelectList cats = new MultiSelectList(Categories, "ProductSubCategoryId", "Name", model.Categorias);
            ViewData["cats"] = cats;
            
            // Devolvemos la vista.
            return View(model);
        }

        //**************************************************************************
        //Nombre: [HttpPost]RecoveryPswd()
        //Descripción: Envia la clave del usuario al su correo electrónico.
        //@params [LogOnModel model, string returnUrl]
        //**************************************************************************
        [HttpPost]
        public ActionResult RecoveryPswd(LogOnModel model, string returnUrl)
        {
            try
            {
                // Inentamos enviar la clave del usuario a su dirección de correo.
                string returncode = MembershipService.RecoveryPassword(model.UserName);

                // Si todo ha ido bien.
                if (returncode == "OK")
                {
                    ViewData["Result"] = "Su clave ha sido enviada a su dirección de correo.";
                }
                else
                {
                    // Algo ha fallado, mostramos mensaje en la vista de respuesta RecoveryPswd.aspx
                    ViewData["Result"] = "La dirección de correo introducida figura en nuestra base de datos.";
                }

                // Devolvemos la vista.
                return View();
            }
            catch (Exception e)
            {
                // Si algo ha fallado mostramos el mensaje en la vista
                 ViewData["Result"] = e.Message;
                return View();
            }
        }

    }
}
